// feature collection

struct Feature_Bayes
{
    // tag non/interactions
    static const int TP = 1;
    static const int TN = 0;

    // probabilities
    Vec < double > probs_t, probs_n;
    Vec < double > probs_t_train_bay, probs_n_train_bay;
    Vec < double > probs_t_test_bay,probs_n_test_bay;

    // number of features
    int nfeat;
    int nneg;
    int npos;
    int npos_train_bay;
    int nneg_train_bay;
    int npos_test_bay;
    int nneg_test_bay;

    // labels
    Hash < string, int > href;
    Hash < string, int > href_train;
    Hash < string, int > href_test;

    
 
    // path
    string path;

    // construct
    void construct(int nfeat, string path)
    {
        // features
        this->nfeat = nfeat;
        this->path  = path;

        // loop
        probs_t.resize(nfeat);
        probs_t.fill(1.0);
        probs_n.resize(nfeat);
        probs_n.fill(1.0);

        // update reference
        href.clear();

        // load negative and positive
        npos = add_reference(path + "/positive.txt", href, TP);
        nneg = add_reference(path + "/negative.txt", href, TN);
        //cout<<**<<","<<npos<<","<<nneg<<endl;
    }

    // make id
    string getid (string ida, string idb)
    {
        // get id
        string id = ida + " " + idb;
        if (ida > idb)
            id = idb + " " + ida;
        return id;
    }

    // load set
    int add_reference(string fname, Hash < string, int> &lhash, int type)
    {
        // count
        int nentry = 0;

        // load file
        Vec < string > lida, lidb;
        Format::readPairs(fname, lida, lidb);
        for (int i = 0; i < lida.size(); i++)
        {
            // get id
            string id = getid(lida[i], lidb[i]);
            if (lhash.get(id) == -1)
            {
                lhash.insert(id, type);
                nentry++;
            }
        }

        // return size
        return nentry;
    }

    // train parameters
    void train(Feature_Table& f, int n)
    {
        // reset
        double n_t = 0.0;
        double n_n = 0.0;

        // count
        for (int i = 0; i < n; i++)
        {
            for (int j = 0; j < nfeat; j++)
            {
                // get feature value
                bool evidence = f.ftable[i][j];

                // check
                if (!evidence)
                    continue;

                // get type
                int type = href.get(f.fid[i]);

                // skip entries not included in the training set
                if (type == -1)
                    continue;

                // count true positive
                
                if (type == TP)
                {
                    probs_t[j]++;
                    n_t++;
                } else {
                    probs_n[j]++;
                    n_n++;
                }
            }
        }

        /*/ normalize
        for (int i = 0; i < nfeat; i++)
        {
            if (n_t > 0.0)
                probs_t[i] /= n_t;
            if (n_n > 0.0)
                probs_n[i] /= n_n;
        }*/
    }

    // predict
    double predict(Vec < bool >& line)
    {
        // constant prior
        double hypo_t = 1.0;
        double hypo_n = 1.0;
        for (int i = 0; i < nfeat; i++)
        {
            // check if evidence is available
            bool evidence = line[i];
            if (!evidence)
                continue;

            // add up
            hypo_t *= probs_t[i];
            hypo_n *= probs_n[i];
            //cout<<probs_t[i]<<","<<probs_n[i]<<endl;
        }
        
        // check
        if (hypo_t <= 0.0 || hypo_n <= 0.0)
            Msg::error("", "Hypothesis with zero probability.");

        // return
        return int (100.0 * hypo_t / hypo_n) / 100.0;
    }

    // show table
    void show(Feature_Table& f, int n)
    {
        // count
        for (int i = 0; i < n; i++)
        {
            int type = href.get(f.fid[i]);
            if (type == -1)
                continue;
            cout << f.fid[i] << " ";
            for (int j = 0; j < nfeat; j++)
	     cout <<f.ftable[i][j] << " ";
             cout << type << " ";
             cout << endl;
        }
    }
    
    // show probabilities
    void show_probs()
    {
        // positive
        for (int i = 0; i < nfeat; i++)
            cout << "Feature (pos.) " << i << " : " << int(probs_t[i] * 100.0) / 100.0 << endl;
        cout << endl;
        
        // negative
        for (int i = 0; i < nfeat; i++)
            cout << "Feature (neg.) " << i << " : " << int(probs_n[i] * 100.0) / 100.0 << endl;
        cout << endl;
    }

  void construct_train_bay(int nfeat, string path, int n)
    {
        
      int m[2];
	  // features
        this->nfeat = nfeat;
        this->path  = path;

        // loop
        probs_t_train_bay.resize(nfeat);
        probs_t_train_bay.fill(1.0);
        probs_n_train_bay.resize(nfeat);
        probs_n_train_bay.fill(1.0);

        // update reference
        href_train.clear();
	//cout <<path <<"/gs_train_set_" <<Convert::toString(n) <<".txt"<<endl;
        // load negative and positive
        add_reference_bay(path + "/gs_train_set_" + Convert::toString(n) + ".txt", href_train, m );

	npos_train_bay = m [0];
	nneg_train_bay = m [1];
        //cout<<"**"<<","<<npos_train_bay<<","<<nneg_train_bay<<endl;
    }

     // load set
       void add_reference_bay(string fname, Hash < string, int> &lhash, int* m)
    {
        // count
      m[0] = 0;
      m[1] = 0;
      // check
        
        // load file
        Vec < Vec < string > > fcontent;
        Format::readTable(fname, fcontent);

        // list
        
        for (int i = 0; i < fcontent.size(); i++)
        {
            // check minimum length
            int lng = fcontent[i].size();
            if (lng < 2)
                continue;

            // get score
            bool type; 
            if (lng >= 3)
                type = Convert::toDbl(fcontent[i][2]);
             // load file
          
           
            // get names
            string ida = fcontent[i][0];
            string idb = fcontent[i][1];

            string id = getid(ida, idb);
            if (lhash.get(id) == -1)
            {
                lhash.insert(id, type);
		if (type == 1 ) m[0] ++ ;
		else m[1] ++;
            }
        } 
    }



    void train_bay(Feature_Table& f, int n)
    {
        // reset
        double n_t = 0.0;
        double n_n = 0.0;

        // count
        for (int i = 0; i < n; i++)
        {
            for (int j = 0; j < nfeat; j++)
            {
                // get feature value
                bool evidence = f.ftable[i][j];

                // check
                if (!evidence)
                    continue;

                // get type
                int type = href_train.get(f.fid[i]);

                // skip entries not included in the training set
                if (type == -1)
                    continue;

                // count true positive
                
                if (type == TP)
                {
                    probs_t_train_bay[j]++;
                    n_t++;
                } 
				else 
				{
                    probs_n_train_bay[j]++;
                    n_n++;
                }
            }
        }

        /*/ normalize
        for (int i = 0; i < nfeat; i++)
        {
            if (n_t > 0.0)
                probs_t[i] /= n_t;
            if (n_n > 0.0)
                probs_n[i] /= n_n;
        }*/
    }

    // show table
    void show_bay(Feature_Table& f, int n)
    {
        // count
        for (int i = 0; i < n; i++)
        {
            int type = href_train.get(f.fid[i]);
            if (type == -1)
                continue;
            cout << f.fid[i] << " ";
            for (int j = 0; j < nfeat; j++)
				cout <<f.ftable[i][j] << " ";
			cout << type << " ";
			cout << endl;
        }
    }    
            

  void construct_test_bay(int nfeat, string path, int n)
    {
        // features
        this->nfeat = nfeat;
        this->path  = path;

        // loop
        probs_t_test_bay.resize(nfeat);
        probs_t_test_bay.fill(1.0);
        probs_n_test_bay.resize(nfeat);
        probs_n_test_bay.fill(1.0);

        // update reference
        href_test.clear();

        // load negative and positive
        npos_test_bay = add_reference(path + "/gs_pos_set_" + Convert::toString(n) + ".txt", href_test, TP);
        nneg_test_bay = add_reference(path + "/gs_neg_set_" + Convert::toString(n) + ".txt", href_test, TN);
        //cout<<**<<","<<npos<<","<<nneg<<endl;
    }


  void show_probs_train_bay()
    {
        // positive
        for (int i = 0; i < nfeat; i++)
            cout << "Feature (pos.) " << i << " : " << int(probs_t_train_bay[i] * 100.0) / 100.0 << endl;
        cout << endl;
        
        // negative
        for (int i = 0; i < nfeat; i++)
            cout << "Feature (neg.) " << i << " : " << int(probs_n_train_bay[i] * 100.0) / 100.0 << endl;
        cout << endl;
    }
 
    double predict_bay(Vec < bool >& line)
    {
        // constant prior
        double hypo_t = 1.0;
        double hypo_n = 1.0;
        for (int i = 0; i < nfeat; i++)
        {
            // check if evidence is available
            bool evidence = line[i];
            if (!evidence)
                continue;

            // add up
            hypo_t *= probs_t_train_bay[i];
            hypo_n *= probs_n_train_bay[i];
            //cout<<probs_t[i]<<","<<probs_n[i]<<endl;
        }
        
        // check
        if (hypo_t <= 0.0 || hypo_n <= 0.0)
            Msg::error("", "Hypothesis with zero probability.");

        // return
        return int (100.0 * hypo_t / hypo_n) / 100.0;
    }


};

