// feature collection
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
struct Feature_Table
{
    // features: evidence yes/no
    Vec <  Vec < bool > > ftable;
    
    // features: full score
    Vec <  Vec < double > > fscore;

    //GS
    //Vec <> 
    
    // name of protein pair
    Vec < string > fid;
    
    // feature name
    Vec < string > fname;

    // number of features
    int nfeat;

    // index
    Hash < string, int > hindex;

    // construct
    void construct (int nfeat)
    {
        // number of features
        this->nfeat = nfeat;

        // reset content
        fid.clear();
        ftable.clear();
        fscore.clear();
        
        // reset index
        hindex.clear();
        
        // reset name
        fname.resize(nfeat);
        fname.fill("");
    }
    
    // make id
    string getid (string ida, string idb)
    {
        string id = ida + " " + idb;
        if (ida > idb)
            id = idb + " " + ida;
        return id;
    }
    
    // get index
    int getindex (string ida, string idb)
    {
        // get id
        string id = getid(ida, idb);

        // add new content
        if (hindex.get(id) == -1)
        {
            // create new index
            hindex.insert(id, hindex.size());

            // initialize with no evidence available
            Vec < bool > fline;
            fline.resize(nfeat);
            fline.fill(false);
            ftable.push_back(fline);

            // initialize with no evidence available
            Vec < double > fline_score;
            fline_score.resize(nfeat);
            fline_score.fill(0.0);
            fscore.push_back(fline_score);

            // add
            fid.push_back(id);
        }
        
        // return
	//cout <<hindex.get(id)<<endl;
        return hindex.get(id);
    }
    
    // add file content
  void addfeature (string filename, int feature, double minthreshold = 0.0, double maxthreshold = LARGE)
    {
        // check
        if (feature >= nfeat)
        {
            Msg::write ("Feature out of range.");
            return;
        }
        
        // skip files
        if (!File::exists(filename))
            return;
        
        // load file
        Vec < Vec < string > > fcontent;
        fname[feature] = Format::readTable(filename, fcontent);

        // list
        int nentry = 0;
        for (int i = 0; i < fcontent.size(); i++)
        {
            // check minimum length
            int lng = fcontent[i].size();
            if (lng < 2)
                continue;

            // get score
            double score = 1.0;
            if (lng >= 3)
                score = Convert::toDbl(fcontent[i][2]);

            // check threshold
            if (score < minthreshold || score > maxthreshold)
                continue;
                    
            // get names
            string ida = fcontent[i][0];
            string idb = fcontent[i][1];
            
            // skip
            //if (ida == idb)
            //    continue;
            
            // get index and update entry
            int index = getindex(ida, idb);
            
            // set evidence/feature availability
            ftable[index][feature] = true;
            fscore[index][feature] = score;
	    
            //cout <<feature<<","<<index<<","<<ftable[index][feature]<<","<<fscore[index][feature]<<endl;

            // counter
	    
            nentry++;
            
            
        }
        
        // log
        Msg::write("%i different pairs loaded for feature %i.", nentry, feature);
	//for (int i = 0 ; i < nentry ; i ++ )
	//cout <<fid[i]<<endl;
    }
  
  //
    
    // add file content
  void addgs (string filename, int feature, bool bl, double minthreshold = 0.0, double maxthreshold = LARGE)
    {
        // check
        if (feature >= nfeat)
        {
            Msg::write ("Feature out of range.");
            return;
        }
        
        // skip files
        if (!File::exists(filename))
            return;
        
        // load file
        Vec < Vec < string > > fcontent;
        fname[feature] = Format::readTable(filename, fcontent);

        // list
        int nentry = 0;
        for (int i = 0; i < fcontent.size(); i++)
        {
            // check minimum length
            int lng = fcontent[i].size();
            if (lng < 2)
                continue;

            // get score
            double score = 1.0;
            if (lng >= 3)
                score = Convert::toDbl(fcontent[i][2]);

            // check threshold
            if (score < minthreshold || score > maxthreshold)
                continue;
                    
            // get names
            string ida = fcontent[i][0];
            string idb = fcontent[i][1];
            
            // skip
            //if (ida == idb)
            //    continue;
            
            // get index and update entry
            int index = getindex(ida, idb);
            
            // set evidence/feature availability
            ftable[index][feature] = bl;
            fscore[index][feature] = score;
	    
            //cout <<feature<<","<<index<<","<<ftable[index][feature]<<","<<fscore[index][feature]<<endl;

            // counter
            nentry++;
            
            
        }
        
        // log
        //Msg::write("%i entries loaded for GS all positive or GS train 4 positive or negative  %i.", nentry, feature);
    }



};
