// log on screen
#define LG_ON

// include
#include "library/interface.h"

// spring specific
#include "bayes/feature_table.h"
#include "bayes/feature_bayes.h"

// bayes for interaction prediction
struct ModSpring_Bayes
{
    // storage
    Storage store;

    // output file
    fstream ftrain;

    // construct
    void construct (string path)
    {
        // number of features
        int nfeat  = 5;

        // loading features
        Msg::write("Loading features...");

        // load features for training
        Feature_Table train;
        train.construct(nfeat);
        
        // add spring threading score to classifier
        //train.addfeature(path + "/feature_0.txt", 0, 0.25);
	//cout<<train.fname[0]<<endl;  
        
        // load other features
	for (int i = 4; i < 5; i++)
	train.addfeature(path + "/feature_" + Convert::toString(i) + ".txt", i);

        // loading features
        Msg::write("Loading reference...");

        // data size
        int ndata = train.ftable.size();
        //cout <<train.fid[34]<<","<<train.ftable[34][4]<<endl;
        //for (int i = 0; i <= 40; i++)
	// cout <<train.fid[i]<<","<<train.ftable[i][4]<<endl;

        // separate
        int nhalf = ndata;

        // loading features
        Msg::write("Initializing network training on %i out of %i interactions...", nhalf, ndata);

        // construct classifier
        Feature_Bayes ba;
        ba.construct(nfeat, path);
        ba.train(train, nhalf);

        // plot table
        //ba.show(train, train.ftable.size());
        ba.show_probs();

        // log
        Msg::write("Network loaded.");

        // arrays
        Vec < int >    lref;
        Vec < string > lid;
        Vec < double > lcon;

	//cout << ba.href[0]<<endl;
        // get interface size
        for (int i = 0; i < ndata; i++)
        {
            // check if its in set
            string id = train.fid[i];
            int type = ba.href.get(id);
            //cout << id << " "<< type << endl;
            if (type == -1)
                continue;

            // confidence
            cout <<train.fid[i] << endl;
            double confidence = ba.predict(train.ftable[i]);

            // add to list
            lid.push_back(id);
            lcon.push_back(confidence);
            lref.push_back(type);
        }

        // make index
        Vec < int > lindex;
        //cout <<lid.size()<<endl;
        lindex.resize(lid.size());
        //cout<<lindex.size()<<endl;
        for (int i = 0; i < lindex.size(); i++)
            lindex[i] = i;

        // sort
        //for (int i = 0; i < lcon.size(); i++)
	//cout <<lindex[i]<<","<<lid[i]<<","<<lcon[i]<<endl; 
        Lib::sort_greater(lcon, lindex);
        //for (int i = 0; i < lcon.size(); i++)
	//cout <<lindex[i]<<","<<lid[i]<<","<<lcon[i]<<endl;

        // log
        Msg::write("Prediction completed for %i with %i/%i pairs.", lid.size(), ba.npos, ba.nneg);

        // make output file
        fstream froc;
        froc.open ("roc.txt", ios_base::out);

        // predict
        double maxmcc = 0.0;
        double maxfp  = 0.0;
        double maxtp  = 0.0;
        double maxcon = 0.0;
        double tp = 0, fp = 0;
	//cout << "***************"<<Feature_Bayes::TP <<endl;
        for (int i = 0; i < lindex.size(); i++)
        {
            // comparison
	  //cout << lid[lindex[i]] <<","<<lref [lindex[i]]<<","<<lcon[i]<<","<<Feature_Bayes::TP<<endl;
            if (lref[lindex[i]] == Feature_Bayes::TP)
	       tp++;
            else
                fp++;

            // get mathew correlation coefficiant
            double fn = ba.npos - tp;
            double tn = ba.nneg - fp;
            double mcc = 0.0;
            double denom = (tp+fp)*(tp+fn)*(tn+fp)*(tn+fn);
            if (denom > 0.0)
            {
                mcc = (tp*tn-fp*fn)/sqrt(denom);
                if (mcc >= maxmcc)
                {
                    maxmcc = mcc;
                    maxcon = lcon[i];
                    maxfp  = fp;
                    maxtp  = tp;
                }
            }

            // print
            //if (i % (lindex.size() / 10) == 0 || i == lindex.size() - 1)
                froc << fp / ba.nneg << " " << tp / ba.npos << " " << mcc << endl;
		//cout << lref[lindex[i]] << " "  <<fp <<" " << ba.nneg << " "<< tp <<" "<< ba.npos << endl;
		//cout <<tp<<","<<fp<<endl;

        }

        // final
        froc << maxfp<<" "<<maxtp<<" "<<maxfp / ba.nneg << " " << maxtp / ba.npos << " " << maxmcc << endl;

        // close
        froc.close();

        // count number of predictions
        fstream fout;
        fout.open("interactome.txt", ios_base::out);
        fout << "Please contact aysam.guerler@gmail.com or zhng@umich.edu for questions." << endl;
        fout << "---------------------------------------------------------------------------------" << endl;

        // write header
        fout << "PROTEINS\tBAYES\tZ[AB-MAX]";
        for (int i = 1; i < nfeat; i++)
            fout << "\t" + train.fname[i];
        fout << endl;
        fout << "---------------------------------------------------------------------------------" << endl;

        // write predictions
        int npredicted = 0;
        for (int i = 0; i < ndata; i++)
        {
            // confidence
	  //cout <<ndata<<endl;
            double confidence = ba.predict(train.ftable[i]);
            //cout <<i<<" "<<train.fid[i]<<" "<< confidence <<endl;

            // count
            //cout <<maxcon<<endl;
            if (confidence >= maxcon)
            {

                fout << train.fid[i] << "\t" << confidence << "\t" << int(train.fscore[i][0] * 100.0) << "\t";
                for (int j = 1; j < nfeat; j++)
                    fout << "\t" << train.fscore[i][j];
                fout << endl;
                npredicted++;
            }
        }
        fout << "---------------------------------------------------------------------------------" << endl;
        fout.close();
        
        // log to screen
        cout << endl;
        cout << "Threshold            : " << maxcon << endl;
        cout << "Total predictions    : " << npredicted << endl;
        cout << "MCC on test set      : " << maxmcc << endl;
    }
};

// main 
int main(int n, char* arg[]) 
{ 
	// parameters
	Vec < string > input;

	// load parameters
	for (int i = 0; i < n; i++)
		input.push_back((string) arg[i]);
		
	// result 
	cout << " **************************************************************************" << endl; 
	cout << " *                             SPRING_BAYES                               *" << endl;
	cout << " *                                                                        *" << endl; 
	cout << " * Reference: A. Guerler et al.                                           *" << endl;
	cout << " * Comments on the program? Please contact: aysam.guerler@gmail.com       *" << endl;
	cout << " **************************************************************************" << endl; 
	cout << endl; 
	
	// run 
    if (n < 2)
        Msg::error ("Missing information", "[directory for feature files]");

    // check args
    ModSpring_Bayes spec;
    spec.construct(input[1]);

	// return 
	return 0; 
} 
