// log on screen
#define LG_ON

// include
#include "library/interface.h"

// bayes for interaction prediction
struct ModSpring_Bayes
{
    // storage
    Storage store;

    // output file
    fstream ftrain;

    // make name
    string makename (string ida, string idb)
    {
        string name = ida + "_" + idb;
        if (ida > idb)
            name = idb + "_" + ida;
        return name;
    }

    // construct
    void construct (string path)
    {
        // loading features
        Msg::write("Loading feature...");
        // load features for testing
        Vec < double > lscore;
        Vec < string > lid;
        Hash < string, int > hid;
        
        File f;
        f.open (path + "/feature_2.txt");
        while (f.good())
        {
            // read line
            if (!f.read())
                continue;

            // get name
            string name = makename(f.get(0), f.get(1));
            if (hid.get(name) == -1)
            {
                hid.insert(name, 0);
                lid.push_back(name);
                lscore.push_back(f.getDbl(2));
           }
        }
        f.close();
        
        // sort
        Lib::sort_greater(lscore, lid);
        // loading features
        Msg::write("Loading reference...");

        // reference hash
        Hash < string, int > lref;

        // load positive data
        int npos = 0;
        f.open (path + "/positive.txt");
        while (f.good())
        {
            if (!f.read())
                continue;
                
            string name = makename(f.get(0), f.get(1));
            if (lref.get(name) == -1)
            {
                lref.insert(name, 1);
                npos++;
            }
        }
        f.close();
        
        // load negative data
        int nneg = 0;
        f.open (path + "/negative.txt");
        while (f.good())
        {
            if (!f.read())
                continue;
                
            string name = makename(f.get(0), f.get(1));
            if (lref.get(name) == -1)
            {
                lref.insert(name, 0);
                nneg++;
            }
        }
        f.close();
        

        // log
        cout << "Total number of negative and postive entries : " << npos << " / " << nneg << endl;

        // make output file
        fstream froc;
        froc.open ("roc.txt", ios_base::out);

        // predict
        int navail = 0;
        double maxmcc = 0.0;
        double maxfp  = 0.0;
        double maxtp  = 0.0;
        double maxcon = 0.0;
        double tp = 0, fp = 0;
	for (int i = 0; i < lid.size(); i++)
        {
            // check if its in set
	   int ref = lref.get(lid[i]);
            if (ref == -1)
                continue;

            // count
            navail++;

            // comparison
            if (ref == 1)
	      {    tp++;
              }            
	    else
              {   fp++;
	      }                                      
          // get mathew correlation coefficiant
            double fn = npos - tp;
            double tn = nneg - fp;
            double mcc = 0.0;
            double denom = (tp+fp)*(tp+fn)*(tn+fp)*(tn+fn);
            if (denom > 0.0)
            {
                mcc = (tp*tn-fp*fn)/sqrt(denom);
                if (mcc >= maxmcc)
                {
                    maxmcc = mcc;
                    maxcon = lscore[i];
                    maxfp  = fp;
                    maxtp  = tp;
                }
            }

            // print
            //if (i % (lid.size() / 10) == 0 || i == lid.size() - 1)
	    // froc << fp / nneg << " " << tp / npos << " " << mcc << endl;
	    froc << fp / nneg << " " << tp / npos << " " << mcc << endl;
        }

        // final
        froc << maxfp / nneg << " " << maxtp / npos << " " << maxmcc << endl;
        cout << "Performance : " << navail << ", " << maxtp << ", " << maxfp << ", " << maxmcc << endl;
        //froc <<nneg<<","<<npos<<","<<tp<<","<<fp<<","<<tn<<","<<fn<<endl;
        
        // close
        froc.close();

        // count number of predictions
        int npredicted = 0;
        for (int i = 0; i < lid.size(); i++)
        {
            double confidence = lscore[i] + EPSILON;

            // count
            if (confidence > maxcon)
                npredicted++;
        }

        // log to screen
        cout << endl;
        cout << "Confidence threshold : " << maxcon << endl;
        cout << "Total predictions    : " << npredicted << endl;
        cout << "MCC on test set      : " << maxmcc << endl;
    }
};

// main 
int main(int n, char* arg[]) 
{ 
	// parameters
	Vec < string > input;

	// load parameters
	for (int i = 0; i < n; i++)
		input.push_back((string) arg[i]);
		
	// result 
	cout << " **************************************************************************" << endl; 
	cout << " *                             SPRING_BAYES                               *" << endl;
	cout << " *                                                                        *" << endl; 
	cout << " * Reference: A. Guerler et al.                                           *" << endl;
	cout << " * Comments on the program? Please contact: aysam.guerler@gmail.com       *" << endl;
	cout << " **************************************************************************" << endl; 
	cout << endl; 
	
	// run 
    if (n < 2)
        Msg::error ("Missing information", "[directory for feature files]");

    // check args
    ModSpring_Bayes spec;
    spec.construct(input[1]);

	// return 
	return 0; 
} 
